from faker import Faker

from pandamonium import pd, np
from pandamonium.exceptions import PandamoniumError


class DataFrame(object):
    def __init__(self, length=100, seed=None, language='en_GB'):
        self.language = language
        self.length = length
        self.seed = seed
        self.columns = {}

        self.faker = Faker(self.language)
        self.faker.seed(self.seed)

        self.np_rand = np.random.RandomState(seed=self.seed)

    def _basic_str(self, **kwargs):
        return [self.faker.text(**kwargs) for _ in range(self.length)]

    def _basic_int(self, **kwargs):
        if 'low' not in kwargs:
            # low is needed, if not supplied then default to 0
            kwargs['low'] = 0
            if 'high' not in kwargs:
                # If low is defined but high isn't, then need to supply it also
                kwargs['high'] = 10
        return self.np_rand.randint(size=self.length, **kwargs)

    def _basic_float(self, **kwargs):
        return self.np_rand.uniform(size=self.length, **kwargs)

    def basic_column(self, name, type, **kwargs):
        types = {
            'float': self._basic_float,
            'int': self._basic_int,
            'str': self._basic_str
        }
        if type not in types.keys():
            raise PandamoniumError('Basic column type {} not in {}'.format(type, list(types.keys())))

        self.columns[name] = types[type](**kwargs)
        return self

    def generator_column(self, name, gen):
        """Constructs column from a generator. Assumes that the generator will never be exhausted."""
        self.columns[name] = [next(gen) for _ in range(self.length)]
        return self

    def stats_column(self, name, obj, **kwargs):
        """Constructs column from a scipy.stats style object. Could be a non-scipy.stats object that follows the same API."""
        self.columns[name] = obj.rvs(size=self.length, **kwargs)
        return self

    def date_column(self, name, start=None, end=None, **kwargs):
        if start is None and end is None:
            raise PandamoniumError('Must set start or end')
        if start is not None and end is not None:
            raise PandamoniumError('Must not set both start and end')

        self.columns[name] = pd.to_datetime(pd.date_range(start=start, end=end, periods=self.length, **kwargs))
        return self

    def _df(self):
        df = pd.DataFrame(self.columns)
        return df

    @property
    def df(self):
        return self._df()
