from random import choice

from scipy.stats import poisson, bernoulli

from pandamonium import DataFrame, pd, np

class TestDataFrame(object):
    def test_df_is_created(self):
        df = DataFrame()
        assert df

    def test_df_sets_correct_defaults(self):
        df = DataFrame()
        assert df.language == 'en_GB'
        assert df.length == 100
        assert df.seed is None
        # No columns added yet so should be empty
        assert isinstance(df.columns, dict)
        assert len(df.columns) == 0

    def test_df_sets_correct_options(self):
        LANGUAGE = 'ru_RU'
        LENGTH = 10
        SEED = 24601
        df = DataFrame(length=LENGTH, seed=SEED, language=LANGUAGE)

        assert df.language == LANGUAGE
        assert df.length == LENGTH
        assert df.seed == SEED

    def test_df_returns_dataframe_class(self):
        df = DataFrame()
        assert isinstance(df.df, pd.DataFrame)

    def test_basic_columns(self):
        columns = {
            'i': 'int',
            'f': 'float',
            's': 'str',
        }
        factory = DataFrame(length=10)
        for key, type in columns.items():
            factory.basic_column(key, type)
        df = factory.df

        assert isinstance(df, pd.DataFrame)

        for key in columns:
            assert key in df.columns

        assert 'float' in df['f'].dtype.name
        assert 'int' in df['i'].dtype.name
        assert df['s'].dtype.name == 'object'

    def test_generator_columns(self):
        def generator():
            choices = range(10)  # 0-9
            while True:
                yield choice(choices)

        g = generator()

        fact = DataFrame()
        fact.generator_column('col', g)

        df = fact.df

        assert len(df.columns) == 1
        assert np.all(df.col < 10)
        assert np.all(df.col >= 0)

    def test_stats_columns(self):

        p = poisson(mu=10)
        b = bernoulli(p=0.5)

        fact = DataFrame()
        fact.stats_column('poisson', p).stats_column('bern', b)

        df = fact.df

        assert len(df.columns) == 2
        assert np.all(df.poisson >= 0)
        assert np.all(df.bern >= 0)
        assert np.all(df.bern <= 1)

    def test_date_column(self):

        fact = DataFrame()
        fact.date_column('d1', start='2010/06/06', freq='4H').date_column('d2', end='2019/12/31', freq='1D')

        df = fact.df

        assert len(df.columns) == 2
        assert np.all(df.d1 >= '2010/06/06')
        assert np.all((df.d1 - df.d1.shift()).dropna() == pd.Timedelta(hours=4))

        assert np.all(df.d1 <= '2019/12/31')
        assert np.all((df.d2 - df.d2.shift()).dropna() == pd.Timedelta(days=1))

